;;; @file flasher.s
;;; Flash programmer for Nicolas Welte's 6502-RAMROM in the PET or VIC-20
;;; @author Marko Mkel (msmakela@nic.funet.fi)

;;; Copyright  2003 Marko Mkel
;;;
;;;     This program is free software; you can redistribute it and/or modify
;;;     it under the terms of the GNU General Public License as published by
;;;     the Free Software Foundation; either version 2 of the License, or
;;;     (at your option) any later version.
;;;
;;;     This program is distributed in the hope that it will be useful,
;;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;     GNU General Public License for more details.

;;; This file can be compiled with Andr Fachat's xa as follows.
;;; xa -o flasher-pet.prg flasher.s
;;; xa -DVIC -o flasher-vic.prg flasher.s

#ifdef VIC
dst	=$fb			; destination pointer
bend	=$2d			; end of BASIC text pointer
bstart	=$1201			; start of BASIC text
flash	=$a000			; start of flash ROM area
#else
dst	=$c7			; destination pointer
bend	=$2a			; end of BASIC text pointer
bstart	=$401			; start of BASIC text
flash	=$9000			; start of flash ROM area
#endif
chrout	=$ffd2			; display a character
npages	=bend			; number of pages to copy

	.word bstart
*=	bstart
	.word nextln
	.word 2003
	.byte $9e
	.byte $30 + (start / 1000)
	.byte $30 + ((start - (start / 1000 * 1000)) / 100)
	.byte $30 + ((start - (start / 100 * 100)) / 10)
	.byte $30 + (start - (start / 10 * 10))
	.byte $3a, $a2		; new
	.byte 0
nextln	.word 0

	;; check the start address
start	ldy #<(mstart-msgs)	; possible error => start address outside flash
	lda prg			; the LSB of the load address must be 0
	bne msg
	sta dst
	lda prg+1		; check the MSB
	cmp #>flash
	bcc msg
	sta dst+1
	;; check the end address
startok	ldy #<(mdata-msgs)	; possible error => length not $xx00
	lda bend
	cmp #<(prg+2)
	bne msg
	lda bend+1
	sbc #>(prg+2)		; C=1
	bcc msg
	beq msg
	sta npages
	ldy #<(mend-msgs)
	clc
	adc dst+1
	bcc endok
	bne msg
	;; display DIP switch information and wait for keypress
endok	ldy #<(mwait-msgs)
	jsr msg
	sei
	;; wait for keypress
#ifdef VIC
	sta $9120		; A=0
waitkey	ldy $9121
#else
waitkey	inc $e810
	ldy $e812
#endif
	iny
	beq waitkey
	;; copy the data
	ldy #0
loop
#ifdef VIC
#else
	lda dst+1		; skip the I/O area at $e800-$efff
	cmp #$e8
	bcc do
	cmp #$f0
	bcc dont
do
#endif
	jsr copy128
	jsr copy128
dont	inc srch
	inc dst+1
	dec npages
	bne loop
	cli
	rts

	;; display message
msg	lda msgs,y
	beq msg0		; end of error message
	jsr chrout
	iny
	bne msg			; branch always
msg0	rts

msgs	
;;; start address mismatch
mstart	.byte "START<>$XX00 OR <$"
#if flash < $a000
	.byte $30 + (flash / 4096)
#else
	.byte $41 - 10 + (flash / 4096)
#endif
	.byte "000", 0
;;; length mismatch
mdata	.byte "WANT 256*N BYTES", 0
;;; end address mismatch
mend	.byte "END>$FFFF", 0
;;; dip switch configuration
#ifdef VIC
mwait	.byte "SET DIP=111(11)XXYY &", 13, "PRESS A KEY", 0
#else
mwait	.byte "SET DIP=11(111)XXYY AND PRESS A KEY", 0
#endif

	;; unlock the device and copy 128 bytes of data
copy128	lda #$aa
	sta $d555
	lda #$55
	sta $aaaa
	lda #$a0
	sta $d555		; write protect disable
	;; device unlocked, now copy 128 bytes
	ldx #128
copy1
srch = * + 2
	lda prg+2,y
	sta (dst),y
	iny
	dex
	bne copy1
	;; wait 20 ms after writing 128 bytes
	txa
	ldx #$11
w20ms	adc #1
	bne w20ms
	dex
	bne w20ms
#ifdef VIC
	lda #$0f		; C=1 after the loop
	adc $900f
	sta $900f		; change the background colour
#else
	lda #$80
	eor $8000		; toggle a cursor at the top left
	sta $8000
#endif
	rts

	;; the data to be written is appended here
prg
